#!/usr/bin/perl -w 
package dbm;

use strict;

=pod 

=head1 NAME

dbm - encapsulates BerkeleyDB and MLDBM and exposes methods to go get a hash

=head1 SYNOPSYS

use dbm;

my $read_only_hash = readdb($filename);
my $writeable_hash = writedb($filename);

=head1 DESCRIPTION

Does the opening (or dying on failure with an informative message) and
tying of hash for you.  Returns a hash ref. 

If BerkeleyDB and MLDBM are not available, then methods simply return
a new hashref, and databases will be built in memory.  Clients can
query $dbm::BERKELEY to see if BerkeleyDB is being used.

=cut


use Carp;
use FileHandle;
require Exporter;
our @ISA = qw(Exporter);

our @EXPORT = qw(writedb readdb closedb);

our $BERKELEY;
BEGIN {
    $BERKELEY=0;
    eval {
	require BerkeleyDB;
	require MLDBM;
	import BerkeleyDB;
	import MLDBM qw(BerkeleyDB::Hash Storable);
	$BERKELEY = 1;
    };
#    warn "importing BerkeleyDB and MLDBM warned: $@\n" if $@;
    if ($BERKELEY) {
	require BerkeleyDB;
	require MLDBM;
	import BerkeleyDB;
	import MLDBM qw(BerkeleyDB::Hash Storable);
    }
}

my %dbs = ();
my %in_memory_dbs = ();

sub writedb {
    my $name = shift;
    if ($BERKELEY) {
	unlink($name);
	no strict "subs";
	my %h = ();
	$dbs{$name} = tie(%h, "MLDBM",
			  -Filename => $name,
			  -Flags    => DB_CREATE)
	    or die "cannot write [$name]: $! $BerkeleyDB::Error\n";
	return \%h;
    } else {
	if (-e $name) {
	    if (-r _) {
		eval {
		    our $VAR1;
		    require $name;
		    $in_memory_dbs{$name} = $VAR1;
		};
		die "cannot read [$name]: $!$@\n" if $@;
	    } else {
		die "cannot read [$name]\n";
	    }
	} else {
	    my $fh = FileHandle->new($name, "w");
	    die "cannot write [$name]: $!\n" unless $fh;
	    $dbs{$name} = $fh;
	    $in_memory_dbs{$name} = {};
	}
	return $in_memory_dbs{$name};
    }
}

sub readdb {
    my $name = shift;
    if ($BERKELEY) {
	my %h = ();
	no strict "subs";
	$dbs{$name} = tie (%h, "MLDBM", {-Filename=> "$name",
					 -Flags => DB_RDONLY,
					 -Cachesize => 100000000}) 
	    or die "cannot read [$name]: $! $BerkeleyDB::Error\n";
	return \%h;
    } elsif ($in_memory_dbs{$name}) {
	return $in_memory_dbs{$name};
    } else {
	eval {
	    our $VAR1;
	    require $name;
	    $in_memory_dbs{$name} = $VAR1;
	};
	die "cannot read [$name]: $!$@\n" if $@;
	return $in_memory_dbs{$name};
    }
}


sub closedb {
    my $name = shift;
    if ($BERKELEY) {
	confess("Cannot close unknown database [$name]") unless $dbs{$name};
	$dbs{$name}->db_close();
    } elsif ($in_memory_dbs{$name}) {
	my $fh = $dbs{$name};
	$fh ||= FileHandle->new($name, "w");
	die "cannot write [$name]: $!\n" unless $fh;

	use Data::Dumper;
	print $fh Dumper($in_memory_dbs{$name});
	print $fh "\n\n1;\n";
	close $fh;

	delete $in_memory_dbs{$name};
	delete $dbs{$name};
    } else {
	confess "Cannot close unknown database named [$name]";
    }
}

unless (caller()) {
    print "ready.\n" if $BERKELEY;
}

1;
