require File.dirname(__FILE__) + "/general_utilities/general_utilities.rb"
require File.dirname(__FILE__) + "/parsing/parseable.rb"
require File.dirname(__FILE__) + "/parsing/data_point.rb"
require File.dirname(__FILE__) + "/counters.rb"
require File.dirname(__FILE__) + "/threshold.rb"
require File.dirname(__FILE__) + "/error_rate_space.rb"

class DataGroup

  extend Parseable

  attr_reader :data_points

  #< Initialization

  def initialize(data_points)
    @data_points = data_points.sort { |a,b| a.score <=> b.score }
    @total_yes = @data_points.select { |pt| pt.judgement == :yes }.length
    @total_no = @data_points.select { |pt| pt.judgement == :no }.length
  end

  def == (other)
    @data_points == other.instance_eval { @data_points }
    # the other two are derived fields
  end

  def self.parse_from_strings(strings)
    DataGroup.new(DataPoint.parse_from_strings(strings))
  end

  #>

  #< Simple properties

  def same_question_ids?
    data_points.inject(true) { |val, pt| val && pt.question_id == data_points[0].question_id }
  end

  def same_nugget_ids?
    data_points.inject(true) { |val, pt| val && pt.nugget_id == data_points[0].nugget_id }
  end

  def question_id
    if same_question_ids?
      data_points[0].question_id
    else
      raise "Requested question id, but they differ!"
    end
  end

  def nugget_id
    if same_nugget_ids? && same_question_ids?
      data_points[0].nugget_id
    else
      if same_question_ids?
        raise "Requested nugget id, but they differ!"
      else
        raise "Requested nugget id, but even the question ids differ!"
      end
    end
  end

  def counters_at(threshold)
    if threshold.kind_of? Threshold
      counters_at(threshold.cutoff)
    else
      no_right = data_points.select { |pt| pt.score < threshold && pt.judgement == :no }.length
      no_wrong = data_points.select { |pt| pt.score < threshold && pt.judgement == :yes }.length
      Counters.new(no_right, no_wrong, @total_yes - no_wrong, @total_no - no_right)
    end
  end

  #>

  #< Threshold walk

  def each_with_score(step)
    counters = Counters.new(0, 0, @total_yes, @total_no)
    index = 0
    bound = @data_points.last.score + step
    0.step(bound, step) do |new|
      while index < @data_points.length && @data_points[index].score < new do
        counters.step_over!(@data_points[index].judgement)
        index += 1
      end
      yield new, counters
    end
  end

  def lowest_threshold
    Threshold.new(self, 0, Counters.new(0, 0, @total_yes, @total_no))
  end

  # To ensure that all threshold walks are over the same threshold
  # objects.
  memoize :lowest_threshold

  def walk_thresholds
    thresh = lowest_threshold
    while !thresh.last?
      yield thresh
      thresh = thresh.next_definite
    end
    yield thresh
  end

  def thresholds
    unless @thresholds
      ans = []
      walk_thresholds { |th| ans.push(th) }
      @thresholds = ans
    end
    @thresholds
  end

  def intervaled_thresholds
    thresholds.select { |th| th.interval_length > 0 }
  end

  # A collection of the highest likelihood thresholds for
  # every class of error rates.
  def highest_likelihood_thresholds
    ErrorRateSpace.new(self)
  end
  memoize :highest_likelihood_thresholds

  def max_likelihood_threshold(eps_low, eps_high)
    highest_likelihood_thresholds.max_likelihood_threshold(eps_low, eps_high)
  end
  #>

  #< Probabilities

  # The single maximum a posteriori threshold for a particular
  # pair of error rates.
  def best_threshold(eps_low, eps_high = eps_low)
    intervaled_thresholds.max { |th1, th2| th1.unnormalized_posterior_log10(eps_low, eps_high) <=> th2.unnormalized_posterior_log10(eps_low, eps_high) }
  end

  def posterior_thresholds_absolute_relevance(eps_low, eps_high, log10_relevance_cutoff)
    intervaled_thresholds.select { |th| th.unnormalized_posterior_log10(eps_low, eps_high) > log10_relevance_cutoff }
  end

  def posterior_thresholds_relative_relevance(eps_low, eps_high, log10_relative_relevance_cutoff)
    posterior_thresholds_absolute_relevance(eps_low, eps_high, best_threshold(eps_low, eps_high).unnormalized_posterior_log10(eps_low, eps_high) - log10_relative_relevance_cutoff)
  end

#  instrument :posterior_thresholds_relative_relevance
#  instrument :posterior_thresholds_absolute_relevance

  def yes_probability_fixed_epsilons(score, eps_low, eps_high)
    normalization_constant_log10 = posterior_normalization_constant_log10(eps_low, eps_high)
    posterior_thresholds_relative_relevance(eps_low, eps_high, 7).inject(0) { |sum, th| sum + th.yes_probability(score, eps_low, eps_high) * th.posterior_probability(eps_low, eps_high, normalization_constant_log10) }
  end

  instrument :yes_probability_fixed_epsilons

  def posterior_normalization_constant_log10(eps_low, eps_high)
    log10_prob_fixed_epsilon_marginal_threshold(eps_low, eps_high)
  end

#  instrument :posterior_normalization_constant_log10
  memoize :posterior_normalization_constant_log10

  def yes_probability_with_distribution(score, epsilon_distribution)
    epsilon_distribution.integrate_by_relevance(1) { |epsilons| yes_probability_fixed_epsilons(score, epsilons[0], epsilons[1]) }
  end

  instrument :yes_probability_with_distribution

  memoize :yes_probability_with_distribution

  #>
  #< Probabilities of this data

  def generic_probability(counter_method, *args)
    Arithmetic.logspace_sum(intervaled_thresholds.map { |threshold| threshold.counters.send(counter_method, *args) + Math.log10(threshold.interval_length) })
  end

  def log10_prob_fixed_threshold_fixed_epsilon(threshold, eps_low, eps_high = eps_low)
    counters_at(threshold).probability_log10(eps_low, eps_high)
  end

  def log10_prob_fixed_epsilon_marginal_threshold(eps_low, eps_high = eps_low)
    generic_probability(:probability_log10, eps_low, eps_high)
  end

  def log10_prob_marginal_equal_epsilon_marginal_threshold
    generic_probability(:log10_integral_equal_epsilon_to_one)
  end

  def log10_prob_marginal_different_epsilon_marginal_threshold
    generic_probability(:log10_integral_different_epsilon_to_one)
  end

  def log10_prob_biased_coin(bias_yes = (0.0 + @total_yes) / (@total_yes + @total_no))
    Math.log10(bias_yes) * @total_yes + Math.log10(1 - bias_yes) * @total_no
  end
  #>

  #< Plots

  def log10_lin_plot_fixed_epsilon_vs_threshold(step, eps_low, eps_high = eps_low)
    each_with_score(step) { |score, counters| puts "#{score}, #{counters.probability_log10(eps_low, eps_high)}" }
  end

  def log10_lin_plot_marginal_equal_epslion_vs_threshold(step)
    each_with_score(step) { |score, counters| puts "#{score}, #{counters.log10_integral_equal_epsilon_to_one}" }
  end

  def log10_lin_plot_marginal_threshold_vs_equal_epslion(step, epsilon_step)
    epsilon_step.step(1 - epsilon_step, epsilon_step) do |epsilon|
      puts "#{epsilon} #{log10_prob_fixed_epsilon_marginal_threshold(step, epsilon)}"
    end
  end

  def log10_lin_plot_marginal_different_epslion_vs_threshold(step)
    each_with_score(step) { |score, counters| puts "#{score}, #{counters.log10_integral_different_epsilon_to_one}" }
  end

  def log10_lin_lin_plot_marginal_threshold_vs_different_epslion(eps_low_step, eps_high_step=eps_low_step)
    eps_low_step.step(1 - eps_low_step, eps_low_step) do |eps_low|
      STDERR.puts eps_low
      eps_high_step.step(1 - eps_high_step, eps_high_step) do |eps_high|
        puts "#{eps_low} #{eps_high} #{log10_prob_fixed_epsilon_marginal_threshold(eps_low, eps_high)}"
      end
    end
  end

  def log10_lin_lin_plot_equal_epsilon_vs_threshold(step, epsilon_step = 0.01)
    epsilon_step.step(1 - epsilon_step, epsilon_step) do |epsilon|
      each_with_score(step) { |score, counters| puts "#{score}, #{epsilon}, #{counters.probability_log10(epsilon, epsilon)}" }
    end
  end
  #>

end
