require 'test/unit'
require File.dirname(__FILE__) + "/data_set.rb"
require File.dirname(__FILE__) + "/test_helper.rb"

class DataSetTest < Test::Unit::TestCase

  include TestHelper

  def test_simple_parse
    set = DataSet.parse_from_file(SMALL_DATA_FILE) { |pt| 1 }
    assert_equal 1, set.data_groups.length
    assert_equal 6, set.data_groups[0].data_points.length
  end

  def test_one_group_parse
    set = DataSet.parse_from_file(SMALL_DATA_FILE, :one_group)
    assert_equal 1, set.data_groups.length
    assert_equal 6, set.data_groups[0].data_points.length
    assert !set.data_groups[0].same_question_ids?
    assert !set.data_groups[0].same_nugget_ids?
  end

  def X_test_one_group_is_one_group
    seta = DataSet.parse_from_file(SMALL_DATA_FILE) { |pt| 1 }
    setb = DataSet.parse_from_file(SMALL_DATA_FILE, :one_group)
    # TODO: this part fails because procedure equaliy is hard
    assert_equal seta.instance_variable_get(:@point_hasher), setb.instance_variable_get(:@point_hasher)
    assert_equal seta.data_groups[0], DataGroup.parse_from_file(SMALL_DATA_FILE)
    assert_equal seta, setb
  end

  def test_question_parse
    set = DataSet.parse_from_file(SMALL_DATA_FILE, :question)
    assert_equal 4, set.data_groups.length
    assert_equal 6, set.total_points
    set.data_groups.each do |group|
      assert group.same_question_ids?
    end
  end

  def test_nugget_parse
    set = DataSet.parse_from_file(SMALL_DATA_FILE, :nugget)
    assert_equal 5, set.data_groups.length
    assert_equal 6, set.total_points
    set.data_groups.each do |group|
      assert group.same_question_ids?
      assert group.same_nugget_ids?
    end
  end

  def test_new_format_parse
    set = DataSet.parse_from_file(NEW_FORMAT_DATA_FILE, :question)
    assert_equal 5, set.data_groups.length
    assert_equal 214, set.total_points
    set.data_groups.each do |group|
      assert group.same_question_ids?
    end
    assert_equal [1, 2, 2, 28, 181], set.data_groups.map { |gr| gr.data_points.length }.sort
    assert_equal [0, 0, 1, 2, 3], set.data_groups.map { |gr| gr.data_points.select { |pt| pt.judgement == :yes }.length }.sort
    assert_equal [0, 0, 2, 28, 178], set.data_groups.map { |gr| gr.data_points.select { |pt| pt.judgement == :no }.length }.sort
  end

  def test_reallocate
    set = DataSet.parse_from_file(NEW_FORMAT_DATA_FILE, :question)
    assert_equal 5, set.data_groups.length
    assert_equal 214, set.total_points

    newset = set.reallocate(:nugget)
    assert_equal 6, newset.data_groups.length
    assert_equal 214, newset.total_points

    one_group_set = set.reallocate(:one_group)
    assert_equal 1, one_group_set.data_groups.length
    assert_equal 214, one_group_set.total_points

    direct_parse_set = DataSet.parse_from_file(NEW_FORMAT_DATA_FILE, :one_group)
    assert_equal direct_parse_set.only_group.best_threshold(0.02, 0.01).cutoff,
    one_group_set.only_group.best_threshold(0.02, 0.01).cutoff
  end

  def test_reallocate_more
    set = DataSet.parse_from_file(ARTIFICIAL_POINTS, :nugget)
    one_group_set = set.reallocate(:one_group)
    assert_equal 0.35, one_group_set.only_group.best_threshold(0.01, 0.01).cutoff
  end

  def test_epsilon_posterior_smoke
    set = DataSet.parse_from_file(ARTIFICIAL_POINTS, :question)
    set.verbosity = :quiet
    first_answer = set.epsilon_posterior
    assert set.unnormalized_posterior_log10.equal?(set.unnormalized_posterior_log10), "Memoization failure"
  end

  def test_best_epsilon
    set = DataSet.parse_from_file(ARTIFICIAL_POINTS, :question)
    set.verbosity = :quiet
    assert_close_arrays [0.01, 0.01], set.best_epsilon[0..1]
  end

  def test_yes_probability
    set = DataSet.parse_from_file(ARTIFICIAL_POINTS, :one_group)
    set.verbosity = :quiet
    assert_in_delta 0.850304, set.data_groups[0].yes_probability_with_distribution(0.999, set.epsilon_posterior), 1E-4

    known_data_set = DataSet.parse_from_file(LEVEL_TEST_DATA_FILE, :nugget)
    set.verbosity = :quiet
    # NOTE: each line's computed yes-probability grows as the recall
    # score of the corresponding DataPoint.  Note also that input
    # scores required to produce same answers shifted for the new
    # question, and # again for the new nugget.  The source file is
    # test_data/level_test_data_file (or trace through constant)
    assert_close_arrays([0.101084010840108, 0.496205962059621, 0.891327913279133,
                         0.103252032520325, 0.498373983739837, 0.89349593495935,
                         0.105420054200542, 0.500542005420055, 0.895663956639566],
    [DataPoint.new("1001", "1", 0.10), DataPoint.new("1001", "1", 0.15), DataPoint.new("1001", "1", 0.20),
     DataPoint.new("3001", "1", 0.30), DataPoint.new("3001", "1", 0.35), DataPoint.new("3001", "1", 0.40),
     DataPoint.new("3001", "7", 0.50), DataPoint.new("3001", "7", 0.55), DataPoint.new("3001", "7", 0.60)].map { |pt| known_data_set.yes_probability_fixed_epsilons(pt, 0.1, 0.1) })

    assert_equal 0, known_data_set.yes_probability(DataPoint.new("foo", "bar", 0.4))
  end

  def test_expected_f_stubbed_computation
    set = DataSet.parse_from_file(ARTIFICIAL_POINTS, :one_group)
    set.verbosity = :quiet
    set.instance_eval do
      def ok_vital_count_distribution(data_points, nugget_vitality)
        DiscreteProbabilityDistribution.new [0, 1] => 1 # Stub out the answer probability distribtion
      end
    end
    query_judgements = [JudgementRequest.new("1001", { "1" => 0.5 }, "Foo!")]
    nugget_vitalities = { "1" => true }

    assert_equal 1, set.expected_f(query_judgements, nugget_vitalities).f_measure
  end

  def test_expected_f_differently_stubbed_computation
    set = DataSet.parse_from_file(ARTIFICIAL_POINTS, :one_group)
    set.verbosity = :quiet
    set.instance_eval do
      def ok_vital_count_distribution(data_points, nugget_vitality)
        DiscreteProbabilityDistribution.new [0, 2] => 0.5, [2, 1] => 0.5 # Stub out the answer probability distribtion
      end
    end
    query_judgements = [JudgementRequest.new("1001", { "1" => 0.5}, "x" * 400)]
    nugget_vitalities = { "1" => true, "2" => true, "3" => false, "4" => true, "5" => true }

    assert_equal 0.5 * 0.5 + 0.5 * 3.0/8, set.expected_f(query_judgements, nugget_vitalities, 1).f_measure
  end

  def test_expected_f
    set = DataSet.parse_from_file(ARTIFICIAL_POINTS, :one_group)
    set.verbosity = :quiet
    query_judgements = [JudgementRequest.new("1001", { "1" => 0.5 }, "Foo!")]
    nugget_vitalities = { "1" => true }

    assert_in_delta 0.7413, set.expected_f(query_judgements, nugget_vitalities).f_measure, 1E-3
  end

  def test_expected_f_more
    set = DataSet.parse_from_file(TRAINING_AND_JUDGEMENT_DATA, :question)
    set.verbosity = :quiet
    query_judgements = JudgementRequest.parse_from_file(TRAINING_AND_JUDGEMENT_DATA)

    nugget_vitalities_q02 = { "1" => true }
    assert_close_hashes({:beta=>3, :length=>6, :okay_count=>0.0, :allowance=>93.4864448543409, :precision=>0.934864448543409, :f_measure=>0.934864448543409, :num_vital=>1, :vital_count=>0.934864448543409, :recall=>0.934864448543409}, set.expected_f(query_judgements.select { |req| req.qid == "q02" }, nugget_vitalities_q02).fields)

    nugget_vitalities_3001 = { "1" => true, "2" => false, "3" => true, "4" => false, "5" => true, "6" => false, "7" => true }
    expected_f_3001 = set.expected_f(query_judgements.select { |req| req.qid == "3001" }, nugget_vitalities_3001)
    assert_close_hashes({:precision=>0.995996243845894, :length=>7, :num_vital=>4, :vital_count=>0.936724758759008, :beta=>3, :f_measure=>0.253168853718651, :recall=>0.234181189689752, :okay_count=>0.936724758759008, :allowance=>187.344951751802}, expected_f_3001.fields)
  end

  def test_nugget_probabilities
    set = DataSet.parse_from_file(ARTIFICIAL_POINTS, :one_group)
    set.verbosity = :quiet
    query_judgement = JudgementRequest.new("1001", { "1" => 0.99 }, "Foo!")
    assert_close_hashes({"1" => set.yes_probability(DataPoint.new("1001", "1", 0.99))},
                        set.nugget_probabilities(query_judgement))
  end

end
