require File.dirname(__FILE__) + "/parseable.rb"

# One data point in the Nuggeteer inference system.

# A data point corresponds to a response-nugget pair.  The response is
# irrelevant for the training and inference processes, so a DataPoint
# only knows its nugget (by question_id and nugget_id), the Nuggeteer
# score its response got for that nugget, and, if it's a training data
# point, the judgement that resulted.

# The only interesting function of DataPoint is its ability to parse
# data points from text stream inputs.  It understands two "wire
# formats" that have been used by Nuggeteer:

# 1) One line per point, a tab separated sequence of an (integer)
# question id, an (integer) nugget id, a (float) score, and a keyword for
# the judgement from the set "yes", "no", "maybe".

# 2) One line per nugget, a tab separated sequence of an (integer)
# question id, an (integer) nugget id, a (possibly empty) comma
# separated list of (float) scores whose judgements are "yes", and a
# (possibly empty) comma separated list of (float) scores whose
# judgements are "no".  Note that this format requires the presence of
# exactly three tabs, even if some of the score lists are empty.

class DataPoint

  extend LineParseable

  attr_reader :question_id, :nugget_id, :score, :judgement

  def initialize(question_id, nugget_id, score, judgement = nil)
    @question_id = question_id
    @nugget_id = nugget_id
    @score = score
    raise "Illegal judgement: #{judgement}" unless judgement == :yes || judgement == :no || judgement == :maybe || judgement == nil
    @judgement = judgement
  end

  # Parses a single line in either of the wire formats described in
  # the introduction.  If the line happened to be in the first format,
  # returns a single DataPoint representing the information in that
  # line.  If the line happened to be in the second format, returns a
  # (possibly empty) array of DataPoints representing all the
  # information in that line.  If the line not happen to be in
  # any recognized format, returns nil.
  def self.parse_line(line)
    if line =~ /^(#{qidre})\s+([0-9]+)\s+([0-9.e-]+)\s+(yes|no|maybe)\s*$/
#      puts line
      DataPoint.new($1, $2, $3.to_f, $4.to_sym)
    else 
      if line =~ /^(#{qidre})\s([0-9]+)\s([0-9.,e-]*)\s([0-9.,e-]*)\s*$/
        qid = $1
        nid = $2
        yesses = $3
        nos = $4
        yesses.split(/,/).map { |str| DataPoint.new(qid, nid, str.to_f, :yes) } +
          nos.split(/,/).map { |str| DataPoint.new(qid, nid, str.to_f, :no) }
      else
        []
      end
    end
  end

  def ==(other)
    (@question_id == other.question_id and
       @nugget_id == other.nugget_id and
       (@score - other.score).abs < 1E-15 and
       @judgement == other.judgement)
  end

end
