require File.dirname(__FILE__) + "/regex_parseable.rb"
require File.dirname(__FILE__) + "/data_point.rb"

# A request to judge a response on whether it contains various
# nuggets or not.  Consists of the id of the question to which
# this is a response, the string of the response, and a hash
# from nugget ids in the question to scores the response got
# with respect to them.

# The main use of this class is to parse that information from a pipe
# from Nuggeteer and present it to interested parties for further
# processing.  The accepted format is one line per judgement request,
# starting with the string "JUDGE_REQUEST" (quotes for clarity),
# with an optional leading hash mark, followed by a
# whitespace-separated list of an (integer) question id, a string
# representing the nugget and score information, and a response
# string.  The response string may contain whitespace.  The nugget and
# score information string is expected to be a comma-separated list of
# pairs of nugget id and score, the former preceding the latter, and
# being separated from it by the hash arrow "=>" (quotes for clarity).

class JudgementRequest

  extend RegexParseable

  attr_reader :qid, :nugget_scores, :string

  def initialize(qid, nugget_scores, string)
    @qid = qid
    @nugget_scores = nugget_scores
    @string = string
  end

  def self.regex
    /^#?JUDGE_REQUEST\s+(#{qidre})\s+([0-9.e=>,-]+)\s+(.+)$/
  end

  # This is called by RegexParseable to create JudgementRequests out
  # of lines that match the regex.
  def self.parse_regex_groups(qid, nugget_score_string, string)
    JudgementRequest.new(qid, nugget_score_string.split(",").inject( {} ) { |hash, segment| hash[segment.split("=>")[0]] = segment.split("=>")[1].to_f; hash }, string)
  end

  def length
    @string.length # TODO remove spaces?
  end

  def nuggets
    @nugget_scores.keys
  end

  # A data point that represents the score this response got
  # against the given nugget.
  def data_point(nid)
    DataPoint.new @qid, nid, @nugget_scores[nid]
  end

  def data_points
    nuggets.map { |nid| data_point(nid) }
  end

  def ==(other)
    qid == other.qid &&
    nugget_scores == other.nugget_scores &&
    string == other.string    
  end

  # Same format as the parsed one, sans newline
  def to_s
    nugget_score_string = @nugget_scores.keys.inject("") { |str, nid| "#{str},#{nid}=>#{@nugget_scores[nid]}" }[1..-1] # No leading comma
    "JUDGE_REQUEST\t#{@qid}\t#{nugget_score_string}\t#{@string}"
  end

end
