# Relies on the includer having a parse_from_strings method (that
# takes an array of strings and parses), and provides the convenience
# methods parse_from_string, parse_from_stream, and parse_from_file.
#
# Also provides the qidre method, which gives access to the regex that
# identifies qids.
module Parseable

  # The regular expression that selects question ids.
  def qidre
    /[a-zA-Z\d\.]+/
  end

  def parse_from_string(string, *rest, &block)
    parse_from_strings(string.split(/\n/), *rest, &block)
  end

  def parse_from_stream(io, *rest, &block)
    parse_from_strings(io.readlines, *rest, &block)
  end

  def parse_from_file(filename, *rest, &block)
    File.open(filename) do |f|
      parse_from_strings(f.readlines, *rest, &block)
    end
  end

  def parse_from_files(files, *rest, &block)
    parse_from_strings(files.map { |name| File.open(name) { |f| f.readlines } }.flatten, *rest, &block)
  end
end

# Provides parse_from_strings (and all the concomittant convenience
# methods from Parseable), using an underlying parse_line.  The
# parse_line is expected to convert a (one-line) string into either an
# object, an array of objects, or nil.
module LineParseable

  include Parseable

  # Returns a single (flattened) array of all the (non-nil) objects
  # parsed from each of the given strings.
  def parse_from_strings(strings)
    strings.map { |line| parse_line line }.flatten.select { |obj| !obj.nil? }
  end

end
