require 'test/unit'
require File.dirname(__FILE__) + "/response_set.rb"
require File.dirname(__FILE__) + "/nugget_data.rb"

class ResponseSetTest < Test::Unit::TestCase

  TEST_DATA_DIR = File.dirname(__FILE__) + "/test_data"

  def setup
    # Ah, the beauty of immutable objects!
    @@csail1 ||= ResponseSet.new(:judgement_file => TEST_DATA_DIR + "/ciqa.csail1.ciqajudged")
    @@csailif1 ||= ResponseSet.new(:judgement_file => TEST_DATA_DIR + "/ciqa.csailif1.ciqajudged")
    @@csailif2 ||= ResponseSet.new(:judgement_file => TEST_DATA_DIR + "/ciqa.csailif2.ciqajudged")

    @csail1 = @@csail1
    @csailif1 = @@csailif1
    @csailif2 = @@csailif2
  end

  def test_smoke
    assert_equal 30, @csail1.qids.length
    assert_equal 1116, @csail1.length
    assert_equal 1116, @csail1.qid_filter(:all).length
    assert_equal 38, @csail1.qid_filter("26").length
    assert_equal 36, @csail1.qid_filter("47").length
  end

  def test_responses
    assert_equal 0, @csail1.qid_filter("bob").length
    assert_equal(@csail1.qid_filter("27") + @csail1.qid_filter("28"),
                 @csail1.qid_filter(["27", "28"]))
    all_resp = @csail1.qids.inject(ResponseSet.new(:responses => [])) { |run, qid| run + @csail1.qid_filter(qid) }
    assert_equal @csail1, all_resp

    @csail1.each do |r|
      assert_equal r, @csail1.response_by_id(r.question_id, r.response_id)
    end
  end

  def test_common_responses
    assert_equal 631, @csail1.common_responses(@csailif1).length
    assert_equal @csail1.length, (@csail1.common_responses(@csailif1) + @csail1.novel_responses(@csailif1)).length
    assert_equal @csail1.common_responses(@csailif1), @csail1 - @csail1.novel_responses(@csailif1)
  end

  def test_with_nuggets
    assert_equal 117, @csail1.with_nuggets.length
    assert_equal 122, @csailif1.with_nuggets.length
    assert_equal 111, @csailif2.with_nuggets.length
    [ @csail1, @csailif1, @csailif2 ].each do |run|
      assert_equal run.length, run.with_nuggets.length + run.without_nuggets.length
    end
  end

  def test_nuggets_assigned
    assert_equal 140, @csail1.nuggets_assigned.length
    assert_equal ["46-1"], @csail1.nuggets_assigned("46")
    assert_equal ["55-1", "55-10", "55-11", "55-9"], @csail1.nuggets_assigned("55")
    assert_equal(@csail1.nuggets_assigned("43") +
                 @csail1.nuggets_assigned("44") + 
                 @csail1.nuggets_assigned("45"), 
                 @csail1.nuggets_assigned(["43", "44", "45"]))
    # These are different because they were judged as parts of
    # different runs.
    assert_equal 104, @csail1.common_responses(@csailif1).nuggets_assigned.length
    assert_equal 112, @csailif1.common_responses(@csail1).nuggets_assigned.length
  end

  def test_nugget_delta_one_question
    assert_equal(["47-1", "47-10", "47-11", "47-12", 
                  "47-14", "47-17", "47-2", "47-20", 
                  "47-24", "47-25", "47-3", "47-9"],
                 @csail1.nuggets_assigned("47"))
    assert_equal(["47-1", "47-10", "47-11", "47-12",
                  "47-2", "47-24", "47-25", "47-3"],
                 @csailif1.nuggets_assigned("47"))
    assert_equal(["47-9"],
                 @csail1.nuggets_lost(@csailif1, :question => "47"))
    assert_equal(["47-14", "47-17", "47-20", "47-9"],
                 @csail1.nuggets_lost(@csailif1, :question => "47",
                                      :semantics => :whole_run))
    assert_equal ["47-24"], @csail1.nuggets_gained(@csailif1, :question => "47")
    assert_equal [], @csail1.nuggets_gained(@csailif1, :question => "47",
                                            :semantics => :whole_run)
    assert_equal(@csail1.nuggets_lost(@csailif1, :question => "43") +
                 @csail1.nuggets_lost(@csailif1, :question => "44") + 
                 @csail1.nuggets_lost(@csailif1, :question => "45"), 
                 @csail1.nuggets_lost(@csailif1, :questions => ["43", "44", "45"]))
  end

  def test_nugget_delta
    assert_equal 35, @csail1.nuggets_lost(@csailif1, :semantics => :whole_run).length
    assert_equal 32, @csail1.nuggets_lost(@csailif2, :semantics => :whole_run).length
    assert_equal 31, @csailif1.nuggets_lost(@csail1, :semantics => :whole_run).length
    assert_equal 29, @csailif1.nuggets_lost(@csailif2, :semantics => :whole_run).length
    assert_equal 24, @csailif2.nuggets_lost(@csail1, :semantics => :whole_run).length
    assert_equal 25, @csailif2.nuggets_lost(@csailif1, :semantics => :whole_run).length
  end

  def test_nugget_reassignment
    assert_equal [], @csail1.common_responses(@csail1).nuggets_lost(@csail1)
    assert_equal 12, @csail1.common_responses(@csailif1).nuggets_lost(@csailif1).length
    assert_equal 12, @csail1.nuggets_lost(@csailif1, :semantics => :reinterpretation).length
    assert_equal 12, @csail1.common_responses(@csailif1).nuggets_lost(@csailif1.common_responses(@csail1)).length
    assert_equal 20, @csailif1.common_responses(@csail1).nuggets_lost(@csail1).length
    assert_equal 20, @csailif1.nuggets_lost(@csail1, :semantics => :reinterpretation).length
    assert_equal 20, @csailif1.common_responses(@csail1).nuggets_lost(@csail1.common_responses(@csailif1)).length
  end

  def test_per_response_nugget_assignment
    vanishing_response = @csail1.response_by_id("34", "1")
    assert_equal ["34-17"], vanishing_response.nuggets_assigned
    assert @csail1.same_nuggets_assigned?(vanishing_response)
    assert !@csailif1.same_nuggets_assigned?(vanishing_response)
    assert_nil @csailif1.identical_response(vanishing_response)
    
    differently_judged_response = @csailif1.response_by_id("34", "2")
    assert_equal ["34-11"], differently_judged_response.nuggets_assigned
    assert @csailif1.same_nuggets_assigned?(differently_judged_response)
    assert !@csail1.same_nuggets_assigned?(differently_judged_response)
    assert_equal ["34-13"], @csail1.identical_response(differently_judged_response).nuggets_assigned
    assert !@csailif1.identically_judged(@csail1).include?(differently_judged_response)

    poor_response = @csailif2.response_by_id("33", "4")
    assert_equal [], poor_response.nuggets_assigned
    assert @csail1.same_nuggets_assigned?(poor_response)
    assert @csailif1.same_nuggets_assigned?(poor_response)
    assert @csailif2.same_nuggets_assigned?(poor_response)

    identically_judged_response = @csail1.response_by_id("35", "15")
    assert_equal ["35-1"], identically_judged_response.nuggets_assigned
    assert @csail1.same_nuggets_assigned?(identically_judged_response)
    assert @csailif1.same_nuggets_assigned?(identically_judged_response)
    assert_equal ["35-1"], @csailif1.identical_response(identically_judged_response).nuggets_assigned
    assert @csail1.identically_judged(@csailif1).include?(identically_judged_response)

    assert_equal 577, @csail1.identically_judged(@csailif1).length
    assert_equal 637, @csail1.identically_judged(@csailif2).length
  end

  def test_stringing
    assert_equal @csail1, @csail1.common_responses(@csail1)
    assert_equal @csail1, @csail1.common_responses(@csail1).common_responses(@csail1)

    assert_equal 561, @csail1.common_responses(@csailif1).common_responses(@csailif2).length
    assert_equal 561, @csailif1.common_responses(@csail1).common_responses(@csailif2).length
    assert_equal 561, @csailif1.common_responses(@csail1).common_responses(@csailif2.common_responses(@csail1)).length
  end

  def test_trec_f_measure
    @nuggets = NuggetData.parse_from_file(TEST_DATA_DIR + "/06.ciqa_answers.txt")
    assert_kind_of TrecFMeasure, @csail1.trec_f_measure("34", @nuggets)
    # TODO Write some tests for this that check that it acutally comes up
    # with the right answer.
    assert_in_delta 0.2, @csail1.avg_trec_f_measure(@nuggets), 0.1
    assert_in_delta 0.2, @csailif1.avg_trec_f_measure(@nuggets), 0.1
    assert_in_delta 0.2, @csailif2.avg_trec_f_measure(@nuggets), 0.1
  end

end
